function concatStructTSG( hMainFig, tsg_old )
%
% Concatenate some elements of a TSG structure. Mainly :
% - TSG data members (DAYD, DATE, LATX, LONX, SSPS, SSJT, SSTP)
% - Discrete data (_EXT) members (DAYD, DATE, LATX, LONX, SSPS, SSTP) 
%
% Right now, do not concatenate :
% - Calibration coefficent
% - '_HIST'
% - preference
%
% If the concatenanted TSG structures have identical dates, the double
% are supressed.
%
% This function is called by TEST_READ_V2.m

% The function call :
% concatNumTsg - At the end of this file
% concatInt8Tsg - At the end of this file
%
% $Id$

% Retrieve tsg struct
% -------------------
tsg = getappdata( hMainFig, 'tsg_data' );

% Get NO_CONTROL code value
% -------------------------
NO_CONTROL = tsg.qc.hash.NO_CONTROL.code;

% Sort the date in ascending order. The concatenation of the member DAYD
% is made at the end of the function, as these members are used to know
% the size of the arrays
% iOrder is used to sort the members of the strcture
% ----------------------------------------------------------------------
DAYD = [tsg.DAYD; tsg_old.DAYD];
[DAYD, iOrder] = sort(DAYD);

% Concatenate the arrays
% ----------------------
tsg.DATE = [tsg.DATE; tsg_old.DATE];
tsg.DATE = tsg.DATE(iOrder,:);

tsg = concatNumTsg( iOrder, tsg, tsg_old, 'LATX');
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'LONX');
tsg = concatInt8Tsg( iOrder, NO_CONTROL, tsg, tsg_old, 'POSITION_QC');
%
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'SPDC');
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'PRES');
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'FLOW');
%
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'CNDC');
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'CNDC_STD');
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'CNDC_CAL');
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'CNDC_FREQ');
%
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'SSJT');
tsg = concatInt8Tsg( iOrder, NO_CONTROL, tsg, tsg_old, 'SSJT_QC');
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'SSJT_STD');
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'SSJT_CAL');
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'SSJT_FREQ');
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'SSJT_ADJUSTED');
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'SSJT_ADJUSTED_ERROR');
tsg = concatInt8Tsg( iOrder, NO_CONTROL, tsg, tsg_old, 'SSJT_ADJUSTED_QC');
% % concatStrTsg( iOrder, tsg.SSJT_ADJUSTED_HIST, tsg_old.SSJT_ADJUSTED_HIST);
%
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'SSPS');
tsg = concatInt8Tsg( iOrder, NO_CONTROL, tsg, tsg_old, 'SSPS_QC');
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'SSPS_STD');
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'SSPS_CAL');
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'SSPS_ADJUSTED');
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'SSPS_ADJUSTED_ERROR');
tsg = concatInt8Tsg( iOrder,  NO_CONTROL, tsg, tsg_old, 'SSPS_ADJUSTED_QC');
% % concatStrTsg( iOrder, tsg.SSPS_ADJUSTED_HIST, tsg_old.SSPS_ADJUSTED_HIST);
%
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'SSTP');
tsg = concatInt8Tsg( iOrder, NO_CONTROL, tsg, tsg_old, 'SSTP_QC');
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'SSTP_CAL');
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'SSTP_FREQ');
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'SSTP_ADJUSTED');
tsg = concatNumTsg( iOrder, tsg, tsg_old, 'SSTP_ADJUSTED_ERROR');
tsg = concatInt8Tsg( iOrder, NO_CONTROL, tsg, tsg_old, 'SSTP_ADJUSTED_QC');
% % concatStrTsg( iOrder, tsg.SSTP_ADJUSTED_HIST, tsg_old.SSTP_ADJUSTED_HIST);

tsg.DAYD = DAYD;

% Suppress Double. Compare the dates
% ----------------------------------
tsg = supTsgDouble( tsg);

% Save the TSG structure
% ----------------------
setappdata( hMainFig, 'tsg_data', tsg );

% Traitement des donnes Externes (les chantillons discrets)
% -----------------------------------------------------------
tsg.DAYD_EXT          = [tsg.DAYD_EXT;        tsg_old.DAYD_EXT];
tsg.DATE_EXT          = [tsg.DATE_EXT;        tsg_old.DATE_EXT];
tsg.LATX_EXT          = [tsg.LATX_EXT;        tsg_old.LATX_EXT];
tsg.LONX_EXT          = [tsg.LONX_EXT;        tsg_old.LONX_EXT];
tsg.SSPS_EXT          = [tsg.SSPS_EXT;        tsg_old.SSPS_EXT];
tsg.SSPS_EXT_BOTTLE   = strvcat(tsg.SSPS_EXT_BOTTLE, tsg_old.SSPS_EXT_BOTTLE);
tsg.SSPS_EXT_QC       = [tsg.SSPS_EXT_QC;     tsg_old.SSPS_EXT_QC];
tsg.SSPS_EXT_TYPE     = strvcat(tsg.SSPS_EXT_TYPE, tsg_old.SSPS_EXT_TYPE);
tsg.SSPS_EXT_ANALDATE = strvcat(tsg.SSPS_EXT_ANALDATE,tsg_old.SSPS_EXT_ANALDATE);
tsg.SSTP_EXT          = [tsg.SSTP_EXT;        tsg_old.SSTP_EXT];
tsg.SSTP_EXT_QC       = [tsg.SSTP_EXT_QC;     tsg_old.SSTP_EXT_QC];
tsg.SSTP_EXT_TYPE     = strvcat(tsg.SSTP_EXT_TYPE, tsg_old.SSTP_EXT_TYPE);

if ~isempty(tsg.DAYD_EXT)
  [tsg.DAYD_EXT, iOrder] = sort(tsg.DAYD_EXT);
  tsg.DATE_EXT           = tsg.DATE_EXT(iOrder,:);
  tsg.LATX_EXT           = tsg.LATX_EXT(iOrder);
  tsg.LONX_EXT           = tsg.LONX_EXT(iOrder);
  tsg.SSPS_EXT           = tsg.SSPS_EXT(iOrder);
  tsg.SSPS_EXT_QC        = tsg.SSPS_EXT_QC(iOrder);
  tsg.SSPS_EXT_BOTTLE    = tsg.SSPS_EXT_BOTTLE(iOrder,:);
  tsg.SSPS_EXT_TYPE      = tsg.SSPS_EXT_TYPE(iOrder,:);
  tsg.SSPS_EXT_ANALDATE  = tsg.SSPS_EXT_ANALDATE(iOrder,:);
  tsg.SSTP_EXT           = tsg.SSTP_EXT(iOrder);
  tsg.SSTP_EXT_QC        = tsg.SSTP_EXT_QC(iOrder);
  tsg.SSTP_EXT_TYPE      = tsg.SSTP_EXT_TYPE(iOrder,:);
end

% !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! reste  traiter les doubles
  
  setappdata( hMainFig, 'tsg_data', tsg );

end

%% *************************************************************************
function tsg = concatNumTsg( iOrder, tsg, tsg_old, Para)
%
% Concatenate 2 numeric arrays 
%
% Input
% iOrder - Indicies to organise the array in time ascending order
% tsg - First TSG structure
% tsg_old - Second TSG structure
% Para - Member of the structures to concatenate
%
% Output
% tsg - TSG structure
%

% If tsg.Para and tsg_old.Para are empty nothing is done
% ------------------------------------------------------
if isempty(tsg.(Para)) && isempty(tsg_old.(Para))
  return;
  
% If only tsg.PARA is empty, fill it with NaN. The dimension of the final 
% array must must be equal to the sum of the dimension of tsg.DAYD and
% tsg_old.DAYD
% --------------------------------------------------------------------
elseif isempty(tsg.(Para))
  tsg.(Para)      = NaN * ones( size( tsg.DAYD ) );
  
% See preceding comment
% ---------------------
elseif isempty(tsg_old.(Para))
  tsg_old.(Para)  = NaN * ones( size( tsg_old.DAYD ) );
end

% Concatenation
% -------------
tsg.(Para) = [tsg.(Para); tsg_old.(Para)];

% Re-order the struct array
% -------------------------
tsg.(Para) = tsg.(Para)(iOrder);

end

%% *************************************************************************
function tsg = concatInt8Tsg( iOrder, NO_CONTROL, tsg, tsg_old, Para)
%
% Concatenate 2 numeric int8 arrays 
%
% Input
% iOrder - Indicies to organise the array in time ascending order
% NO_CONTROL - Value for no control quality code
% tsg - First TSG structure
% tsg_old - Second TSG structure
% Para - Member of the structures to concatenate
%
% Output
% tsg - TSG structure
%

% If tsg.Para and tsg_old.Para are empty nothing is done
% ------------------------------------------------------
if isempty(tsg.(Para)) && isempty(tsg_old.(Para))
  return;
  
% If only tsg.PARA is empty, fill it with NO_CONTROL. The dimension of the  
% concatenate array must must be equal to the sum of the dimension of 
% tsg.DAYD and tsg_old.DAYD
% --------------------------------------------------------------------
elseif isempty(tsg.(Para))
  tsg.(Para)      = int8(NO_CONTROL) * int8(ones( size( tsg.DAYD )));

% See preceding comment
% ---------------------
elseif isempty(tsg_old.(Para))
  tsg_old.(Para)  = int8(NO_CONTROL) * int8(ones( size( tsg_old.DAYD )));
end

% Concatenation
% -------------
tsg.(Para) = [tsg.(Para); tsg_old.(Para)];

% Re-order the struct array
% -------------------------
tsg.(Para) = tsg.(Para)(iOrder);

end

%%
function tsg = supTsgDouble( tsg)
%
% Suppress measurements made at the same date
% The comparison cannot be made using tsg.DAYD :
% This cannot be done using the Matlab Serial Date format as there can be
% some loss of precision.. Especially when a floating number was written in
% a file.
%
% I use tsg.DATE. Could be simplify by comparing string rather than
% converting DATE with datenum
%
% ------------------------------------------------------------------------

% Convert string array DATE in numerical array
% --------------------------------------------
date = str2num(tsg.DATE);

% Compare adjacent element in an array
% ------------------------------------
date_diff = find( diff( date ) == 0 );
if ~isempty( date_diff )
  tsg.DAYD(date_diff) = [];
  tsg.DATE(date_diff,:) = [];
  
  tsg.LATX(date_diff) = [];
  tsg.LONX(date_diff) = [];
  tsg.POSITION_QC(date_diff) = [];
  
  if ~isempty(tsg.SPDC); tsg.SPDC(date_diff) = []; end
  if ~isempty(tsg.PRES); tsg.PRES(date_diff) = []; end
  if ~isempty(tsg.FLOW); tsg.FLOW(date_diff) = []; end
  if ~isempty(tsg.CNDC); tsg.CNDC(date_diff) = []; end
  if ~isempty(tsg.CNDC_STD); tsg.CNDC_STD(date_diff) = []; end
  if ~isempty(tsg.CNDC_CAL); tsg.CNDC_CAL(date_diff) = []; end
  if ~isempty(tsg.CNDC_FREQ); tsg.CNDC_FREQ(date_diff) = []; end
  
  if ~isempty(tsg.SSJT); tsg.SSJT(date_diff) = []; end
  if ~isempty(tsg.SSJT_QC); tsg.SSJT_QC(date_diff) = []; end
  if ~isempty(tsg.SSJT_STD); tsg.SSJT_STD(date_diff) = []; end
  if ~isempty(tsg.SSJT_CAL); tsg.SSJT_CAL(date_diff) = []; end
  if ~isempty(tsg.SSJT_FREQ); tsg.SSJT_FREQ(date_diff) = []; end
  if ~isempty(tsg.SSJT_ADJUSTED); tsg.SSJT_ADJUSTED(date_diff) = []; end
  if ~isempty(tsg.SSJT_ADJUSTED_ERROR); tsg.SSJT_ADJUSTED_ERROR(date_diff) = []; end
  if ~isempty(tsg.SSJT_ADJUSTED_QC); tsg.SSJT_ADJUSTED_QC(date_diff) = []; end
  % if ~isempty(tsg.SSJT_ADJUSTED_HIST(date_diff,:) = []; end
  
  if ~isempty(tsg.SSPS); tsg.SSPS(date_diff) = []; end
  if ~isempty(tsg.SSPS_QC); tsg.SSPS_QC(date_diff) = []; end
  if ~isempty(tsg.SSPS_STD); tsg.SSPS_STD(date_diff) = []; end
  if ~isempty(tsg.SSPS_CAL); tsg.SSPS_CAL(date_diff) = []; end
  if ~isempty(tsg.SSPS_ADJUSTED); tsg.SSPS_ADJUSTED(date_diff) = []; end
  if ~isempty(tsg.SSPS_ADJUSTED_ERROR); tsg.SSPS_ADJUSTED_ERROR(date_diff) = []; end
  if ~isempty(tsg.SSPS_ADJUSTED_QC); tsg.SSPS_ADJUSTED_QC(date_diff) = []; end
  %  if ~isempty(tsg.SSPS_ADJUSTED_HIST(date_diff,:) = []; end
  
  if ~isempty(tsg.SSTP); tsg.SSTP(date_diff) = []; end
  if ~isempty(tsg.SSTP_QC); tsg.SSTP_QC(date_diff) = []; end
  if ~isempty(tsg.SSTP_CAL); tsg.SSTP_CAL(date_diff) = []; end
  if ~isempty(tsg.SSTP_FREQ); tsg.SSTP_FREQ(date_diff) = []; end
  if ~isempty(tsg.SSTP_ADJUSTED); tsg.SSTP_ADJUSTED(date_diff) = []; end
  if ~isempty(tsg.SSTP_ADJUSTED_ERROR); tsg.SSTP_ADJUSTED_ERROR(date_diff) = []; end
  if ~isempty(tsg.SSTP_ADJUSTED_QC); tsg.SSTP_ADJUSTED_QC(date_diff) = []; end
  % if ~isempty(tsg.SSTP_ADJUSTED_HIST);tsg.SSTP_ADJUSTED_HIST(date_diff,:) = []; end
end
end

%%
function tsg = supTsgExtDouble( tsg)

% Test : suppress measurements made at the same date
% The comparison cannot be made using tsg.DAYD_EXT.
% This cannot be done using the Matlab Serial Date format as there can be
% some loss of precision.
%
% I use tsg.DATE_EXT
% ------------------------------------------------------------------------
if ~isempty( tsg.DATE_EXT )
  
  % Convert string array DATE in numerical array
  % --------------------------------------------
  date = str2num(tsg.DATE_EXT);
  
  % Compare adjacent element in an array
  % ---------------------------------------------
  date_diff = find( diff( date ) == 0 );
  
  if ~isempty( date_diff )
    tsg.DAYD_EXT(date_diff)           = [];
    tsg.DATE_EXT(date_diff,:)         = [];
    tsg.LATX_EXT(date_diff)           = [];
    tsg.LONX_EXT(date_diff)           = [];
    tsg.SSPS_EXT(date_diff)           = [];
    tsg.SSPS_EXT_QC(date_diff)        = [];
    tsg.SSPS_EXT_BOTTLE(date_diff,:)  = [];
    tsg.SSPS_EXT_TYPE(date_diff,:)    = [];
    tsg.SSPS_EXT_ANALDATE(date_diff,:)= [];
    tsg.SSTP_EXT(date_diff)           = [];
    tsg.SSTP_EXT_QC(date_diff)        = [];
    tsg.SSTP_EXT_TYPE(date_diff,:)    = [];
  end
  
end
end



